'From Squeak3.8 of ''5 May 2005'' [latest update: #6665] on 11 October 2006 at 7:03:15 pm'!
SmartSyntaxInterpreterPlugin subclass: #CurlPlugin
	instanceVariableNames: 'curlShare memChunks curlHandleFirst curlHandleLast'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CurlPlugin'!
!CurlPlugin commentStamp: 'dao 10/1/2006 21:43' prior: 0!
CurlHandle is a C-side plugin structure keeping various information associated with the curl_easy_handle and curl_multi_handle.
This information is maintaned by the plugin code. 
Image side requests all services from the plugin passing aLargeIntegerCurlHandle (representing pointer to CurlHandle) to it.
So here and later in the code, convention is as follows:
	CurlHandle		- refers to plugin handle (passed between smalltalk and plugin)
	EasyHandle		- refers to libcurl 'easy' handle (passed between plugin and libcurl)

CurlHandle is organized as a C-structures (see CurlPlugin class >> curlStructDeclaration)
Each download results in a chain of data-buffers filled with data ((see CurlPlugin class >> dataBufferStructDeclaration))
!


!CurlPlugin methodsFor: 'initialize' stamp: 'dao 9/24/2006 17:43'!
initialiseModule

	self export: true.

	"try to init libcurl"
	memChunks := 0.
	((self cCode: 'curl_global_init(CURL_GLOBAL_ALL )') = 0)
		ifFalse: [^false].

	"try to init 'share' part of libcurl"
	((self cCode: 'curlShare = curl_share_init()') = 0)
		ifTrue: [
			self cCode: 'curl_global_cleanup()'.
			^false].
	
	"try to set shared DNS cache option"
	((self cCode: 'curl_share_setopt(curlShare, CURLSHOPT_SHARE, CURL_LOCK_DATA_DNS)') = 0)
		ifFalse: [
			self cCode: 'curl_share_cleanup(curlShare)'.
			self cCode: 'curl_global_cleanup()'.
			^false].
	"hoof,.."

	"set the pointer to the head and tail of linked list to 0"
	curlHandleFirst := 0.
	curlHandleLast := 0.
	^true
	! !

!CurlPlugin methodsFor: 'initialize' stamp: 'dao 9/24/2006 17:23'!
shutdownModule

	self export: true.

	self freeAllCurlHandles.
	self cCode: 'curl_share_cleanup(curlShare)'.
	self cCode: 'curl_global_cleanup()'.
	
	
	^true! !


!CurlPlugin methodsFor: 'primitives multi' stamp: 'dao 9/24/2006 18:26'!
primIsMultiRunning: aLargeIntegerCurlHandle

	"return if we have transfer in progress (otherwise multi handle has installed easy handle in it)"
	| curlHandle  |
	self export: true.
	
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primIsMultiRunning' parameters: #(Oop).

	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	curlHandle isMultiRunningGet = 0 
		ifTrue: [ ^interpreterProxy falseObject ]
		ifFalse: [ ^interpreterProxy trueObject ]
	! !

!CurlPlugin methodsFor: 'primitives multi' stamp: 'dao 9/24/2006 20:26'!
primMultiAttach: aLargeIntegerCurlHandle

	"bind the easy handle to the multi handle, this efficiently will start transfer when curl_multi_perform will be called"
	| curlHandle multiHandle |
	self export: true.
	
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #multiHandle	declareC: 'CURLM* multiHandle'.

	self primitive: 'primMultiAttach' parameters: #(Oop).


	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	
	"multiHandle :=  self cCode: 'curl_multi_init()'.
	multiHandle = 0 ifTrue: [ ^interpreterProxy primitiveFail].
	curlHandle multiHandleSet: multiHandle."

	curlHandle multiStatusSet:  (self cCode: 'curl_multi_add_handle(multiHandleGet(curlHandle), easyHandleGet(curlHandle))') 
		 
	! !

!CurlPlugin methodsFor: 'primitives multi' stamp: 'dao 9/30/2006 11:01'!
primMultiDetach: aLargeIntegerCurlHandle 
	"detach the easy handle from multi handle - this will stop the transfer in progress"

	| curlHandle |
	self export: true.
	self var: #curlHandle declareC: 'curlStruct* curlHandle'.
	self primitive: 'primMultiDetach'
		parameters: #(#Oop ).
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	curlHandle multiStatusSet:  ( self cCode: 'curl_multi_remove_handle(multiHandleGet(curlHandle), easyHandleGet(curlHandle))').
	curlHandle isMultiRunningSet: 0.
	"curlHandle multiHandleFree."! !

!CurlPlugin methodsFor: 'primitives multi' stamp: 'dao 9/24/2006 18:26'!
primMultiInfoRead: aLargeIntegerCurlHandle

	"set status info for the mutli handle ,
	 currently the only useful info it can return - status of the easy transfers bound to this handle
	 we also have only one easy handle associated with the multi handle
	 all this makes decoding very easy - if there is something - return easy curl code
	 otherwise - return nil (this may mean that transfer canceled)"

	| curlHandle msg msgsLeft |
	self export: true.
	
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #msg			declareC: 'CURLMsg* msg'.
	self var: #msgsLeft		declareC: 'int msgsLeft'.

	self primitive: 'primMultiInfoRead' parameters: #(Oop).

	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	
	msg := self cCode: 'curl_multi_info_read( multiHandleGet(curlHandle),   &msgsLeft)'.
	msg = 0 
		ifTrue: ["no news" ]
	 	ifFalse: [ curlHandle easyStatusSet: (self cCode: 'msg->data.result' ) ].
		! !

!CurlPlugin methodsFor: 'primitives multi' stamp: 'dao 10/1/2006 22:41'!
primMultiPerform: aLargeIntegerCurlHandle

	"start or continue the transfer for the multi<-->easy association"
	| curlHandle curlCode  |
	self export: true.
	
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #curlCode		declareC: 'CURLMcode  curlCode'.
	self primitive: 'primMultiPerform' parameters: #(Oop).

	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	"self cCode: 'while(curlCode = curl_multi_perform( multiHandleGet(curlHandle), isMultiRunningPtr(curlHandle) ),
						curlCode == CURLM_CALL_MULTI_PERFORM)'."
	self cCode: 'curlCode = curl_multi_perform( multiHandleGet(curlHandle), isMultiRunningPtr(curlHandle) )'.
	 
	curlHandle multiStatusSet:  curlCode
	! !

!CurlPlugin methodsFor: 'primitives multi' stamp: 'dao 9/24/2006 18:27'!
primMultiStatus: aLargeIntegerCurlHandle

	"get code of the error for the multi interface"
	|  curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primMultiStatus' parameters: #(Oop).
	
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^interpreterProxy signed32BitIntegerFor: (curlHandle multiStatusGet)! !

!CurlPlugin methodsFor: 'primitives multi' stamp: 'dao 9/24/2006 18:27'!
primMultiStrError: aLargeIntegerCurlHandle

	"get text description of the error for the multi interface"
	|  curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primMultiStrError' parameters: #(Oop).
	
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^(self cCode: '(char*) curl_easy_strerror(multiStatusGet( curlHandle))') asString
! !


!CurlPlugin methodsFor: 'primitives easy' stamp: 'dao 9/24/2006 18:23'!
primEasyPerform: aLargeIntegerCurlHandle

	"execute configured easy handle in VM-blocking mode"
	| curlHandle  |
	self export: true.
	
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primEasyPerform' parameters: #(Oop).

	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	curlHandle easyStatusSet: ( self cCode: '(int)curl_easy_perform( easyHandleGet(curlHandle) )' )
! !

!CurlPlugin methodsFor: 'primitives easy' stamp: 'dao 9/24/2006 18:23'!
primEasyReset: aLargeIntegerCurlHandle

	"reset easy handle(clear all libcurl info associated with it)"

	| curlHandle |
	self export: true.
	
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primEasyReset' parameters: #(Oop).

	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	self cCode: 'curl_easy_reset( easyHandleGet(curlHandle) )'.

	! !

!CurlPlugin methodsFor: 'primitives easy' stamp: 'dao 9/24/2006 18:23'!
primEasyStatus: aLargeIntegerCurlHandle

	"get code of the error for the easy interface"
	|  curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyStatus' parameters: #(Oop).
	
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^curlHandle easyStatusGet asOop: Unsigned
! !

!CurlPlugin methodsFor: 'primitives easy' stamp: 'dao 9/24/2006 18:23'!
primEasyStrError: aLargeIntegerCurlHandle

	"get text description of the error for the easy interface"
	|  curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyStrError' parameters: #(Oop).
	
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^(self cCode: '(char*) curl_easy_strerror(easyStatusGet( curlHandle))') asString
! !


!CurlPlugin methodsFor: 'primitives easy-getinfo-string' stamp: 'dao 10/1/2006 15:16'!
get: curlHandle InfoTypeString: infoCode

	| value |
	self inline: true.

	self var: #curlHandle 	declareC: 'curlStruct* curlHandle'.
	self var: #infoCode 					declareC: 'CURLINFO infoCode'.
	self var: #value 					declareC: 'char* value'.
	
	value := 0.
	curlHandle easyStatusSet:  (self cCode: 'curl_easy_getinfo( easyHandleGet(curlHandle), infoCode, &value)').
	^value asString.
	
	! !

!CurlPlugin methodsFor: 'primitives easy-getinfo-string' stamp: 'dao 10/1/2006 15:26'!
primInfoContentType: aLargeIntegerCurlHandle
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primInfoContentType' parameters: #(Oop ).


	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^self cCode: 'getInfoTypeString(curlHandle, CURLINFO_CONTENT_TYPE )'
	! !

!CurlPlugin methodsFor: 'primitives easy-getinfo-string' stamp: 'dao 10/1/2006 15:18'!
primInfoEffectiveUrl: aLargeIntegerCurlHandle
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primInfoEffectiveUrl' parameters: #(Oop ).


	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^self cCode: 'getInfoTypeString(curlHandle, CURLINFO_EFFECTIVE_URL )'
	! !

!CurlPlugin methodsFor: 'primitives easy-getinfo-string' stamp: 'dao 10/1/2006 15:24'!
primInfoFtpEntryPath: aLargeIntegerCurlHandle
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primInfoFtpEntryPath' parameters: #(Oop ).


	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^self cCode: 'getInfoTypeString(curlHandle, CURLINFO_FTP_ENTRY_PATH )'
	! !

!CurlPlugin methodsFor: 'primitives easy-getinfo-string' stamp: 'dao 10/1/2006 15:23'!
primInfoPrivate: aLargeIntegerCurlHandle
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primInfoPrivate' parameters: #(Oop ).


	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^self cCode: 'getInfoTypeString(curlHandle, CURLINFO_PRIVATE )'
	! !


!CurlPlugin methodsFor: 'primitives easy-getinfo-double' stamp: 'dao 9/16/2006 12:05'!
get: curlHandle InfoTypeDouble: infoCode

	| value |
	self inline: true.

	self var: #curlHandle 	declareC: 'curlStruct* curlHandle'.
	self var: #infoCode 					declareC: 'CURLINFO infoCode'.
	self var: #value 					declareC: 'double value'.
	

	value := 0.
	curlHandle easyStatusSet:  (self cCode: 'curl_easy_getinfo( easyHandleGet(curlHandle), infoCode, &value)').
	^interpreterProxy floatObjectOf: value
	! !

!CurlPlugin methodsFor: 'primitives easy-getinfo-double' stamp: 'dao 9/24/2006 18:25'!
primInfoConnectTime: aLargeIntegerCurlHandle
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primInfoConnectTime' parameters: #(Oop ).


	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^self cCode: 'getInfoTypeDouble(curlHandle, CURLINFO_CONNECT_TIME )'
	! !

!CurlPlugin methodsFor: 'primitives easy-getinfo-double' stamp: 'dao 10/1/2006 14:12'!
primInfoContentLengthDownload: aLargeIntegerCurlHandle
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primInfoContentLengthDownload' parameters: #(Oop ).


	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^self cCode: 'getInfoTypeDouble(curlHandle, CURLINFO_CONTENT_LENGTH_DOWNLOAD )'
	! !

!CurlPlugin methodsFor: 'primitives easy-getinfo-double' stamp: 'dao 10/1/2006 14:13'!
primInfoContentLengthUpload: aLargeIntegerCurlHandle
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primInfoContentLengthUpload' parameters: #(Oop ).


	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^self cCode: 'getInfoTypeDouble(curlHandle, CURLINFO_CONTENT_LENGTH_UPLOAD )'
	! !

!CurlPlugin methodsFor: 'primitives easy-getinfo-double' stamp: 'dao 9/24/2006 18:25'!
primInfoNameLookupTime: aLargeIntegerCurlHandle
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primInfoNameLookupTime' parameters: #(Oop ).

	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^self cCode: 'getInfoTypeDouble(curlHandle, CURLINFO_NAMELOOKUP_TIME )'
	! !

!CurlPlugin methodsFor: 'primitives easy-getinfo-double' stamp: 'dao 9/24/2006 18:25'!
primInfoPreTransferTime: aLargeIntegerCurlHandle
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primInfoPreTransferTime' parameters: #(Oop ).


	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^self cCode: 'getInfoTypeDouble(curlHandle, CURLINFO_PRETRANSFER_TIME )'
	! !

!CurlPlugin methodsFor: 'primitives easy-getinfo-double' stamp: 'dao 9/24/2006 18:25'!
primInfoRedirectTime: aLargeIntegerCurlHandle
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primInfoRedirectTime' parameters: #(Oop ).


	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^self cCode: 'getInfoTypeDouble(curlHandle, CURLINFO_REDIRECT_TIME )'
	! !

!CurlPlugin methodsFor: 'primitives easy-getinfo-double' stamp: 'dao 10/1/2006 14:10'!
primInfoSizeDownload: aLargeIntegerCurlHandle
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primInfoSizeDownload' parameters: #(Oop ).


	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^self cCode: 'getInfoTypeDouble(curlHandle, CURLINFO_SIZE_DOWNLOAD )'
	! !

!CurlPlugin methodsFor: 'primitives easy-getinfo-double' stamp: 'dao 10/1/2006 14:04'!
primInfoSizeUpload: aLargeIntegerCurlHandle
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primInfoSizeUpload' parameters: #(Oop ).


	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^self cCode: 'getInfoTypeDouble(curlHandle, CURLINFO_SIZE_UPLOAD )'
	! !

!CurlPlugin methodsFor: 'primitives easy-getinfo-double' stamp: 'dao 10/1/2006 14:10'!
primInfoSpeedDownload: aLargeIntegerCurlHandle
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primInfoSpeedDownload' parameters: #(Oop ).


	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^self cCode: 'getInfoTypeDouble(curlHandle, CURLINFO_SPEED_DOWNLOAD )'
	! !

!CurlPlugin methodsFor: 'primitives easy-getinfo-double' stamp: 'dao 10/1/2006 14:11'!
primInfoSpeedUpload: aLargeIntegerCurlHandle
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primInfoSpeedUpload' parameters: #(Oop ).


	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^self cCode: 'getInfoTypeDouble(curlHandle, CURLINFO_SPEED_UPLOAD )'
	! !

!CurlPlugin methodsFor: 'primitives easy-getinfo-double' stamp: 'dao 9/24/2006 18:26'!
primInfoStartTransferTime: aLargeIntegerCurlHandle
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primInfoStartTransferTime' parameters: #(Oop ).


	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^self cCode: 'getInfoTypeDouble(curlHandle, CURLINFO_STARTTRANSFER_TIME )'
	! !

!CurlPlugin methodsFor: 'primitives easy-getinfo-double' stamp: 'dao 9/24/2006 18:26'!
primInfoTotalTime: aLargeIntegerCurlHandle
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primInfoTotalTime' parameters: #(Oop ).


	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^self cCode: 'getInfoTypeDouble(curlHandle, CURLINFO_TOTAL_TIME )'
	! !


!CurlPlugin methodsFor: 'primitives easy-getinfo-long' stamp: 'dao 10/1/2006 13:55'!
get: curlHandle InfoTypeLong: infoCode

	| value |
	self inline: true.

	self var: #curlHandle 	declareC: 'curlStruct* curlHandle'.
	self var: #infoCode 					declareC: 'CURLINFO infoCode'.
	self var: #value 					declareC: 'long value'.
	

	value := 0.
	curlHandle easyStatusSet:  (self cCode: 'curl_easy_getinfo( easyHandleGet(curlHandle), infoCode, &value)').
	^interpreterProxy signed32BitIntegerFor: (value)
	! !

!CurlPlugin methodsFor: 'primitives easy-getinfo-long' stamp: 'dao 9/24/2006 18:25'!
primInfoFileTime: aLargeIntegerCurlHandle
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primInfoFileTime' parameters: #(Oop ).


	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^self cCode: 'getInfoTypeLong(curlHandle, CURLINFO_FILETIME )'
	! !

!CurlPlugin methodsFor: 'primitives easy-getinfo-long' stamp: 'dao 9/24/2006 18:25'!
primInfoHeaderSize: aLargeIntegerCurlHandle
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primInfoHeaderSize' parameters: #(Oop ).


	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^self cCode: 'getInfoTypeLong(curlHandle, CURLINFO_HEADER_SIZE )'
	! !

!CurlPlugin methodsFor: 'primitives easy-getinfo-long' stamp: 'dao 9/24/2006 18:25'!
primInfoHTTPAuthAvail: aLargeIntegerCurlHandle
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primInfoHTTPAuthAvail' parameters: #(Oop ).


	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^self cCode: 'getInfoTypeLong(curlHandle, CURLINFO_HTTPAUTH_AVAIL )'
	! !

!CurlPlugin methodsFor: 'primitives easy-getinfo-long' stamp: 'dao 9/24/2006 18:25'!
primInfoHTTPConnectCode: aLargeIntegerCurlHandle 
	| curlHandle |
	self export: true.
	self var: #curlHandle declareC: 'curlStruct* curlHandle'.
	self primitive: 'primInfoHTTPConnectCode'
		parameters: #(#Oop ).
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^self cCode: 'getInfoTypeLong(curlHandle, CURLINFO_HTTP_CONNECTCODE )'! !

!CurlPlugin methodsFor: 'primitives easy-getinfo-long' stamp: 'dao 9/24/2006 18:25'!
primInfoOsErrNo: aLargeIntegerCurlHandle
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primInfoOsErrNo' parameters: #(Oop ).


	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^self cCode: 'getInfoTypeLong(curlHandle, CURLINFO_OS_ERRNO   )'
	! !

!CurlPlugin methodsFor: 'primitives easy-getinfo-long' stamp: 'dao 9/24/2006 18:25'!
primInfoProxyAuthAvail: aLargeIntegerCurlHandle
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primInfoProxyAuthAvail' parameters: #(Oop ).


	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^self cCode: 'getInfoTypeLong(curlHandle, CURLINFO_PROXYAUTH_AVAIL  )'
	! !

!CurlPlugin methodsFor: 'primitives easy-getinfo-long' stamp: 'dao 9/24/2006 18:25'!
primInfoRedirectCount: aLargeIntegerCurlHandle
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primInfoRedirectCount' parameters: #(Oop ).


	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^self cCode: 'getInfoTypeLong(curlHandle, CURLINFO_REDIRECT_COUNT )'
	! !

!CurlPlugin methodsFor: 'primitives easy-getinfo-long' stamp: 'dao 9/24/2006 18:26'!
primInfoRequestSize: aLargeIntegerCurlHandle
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primInfoRequestSize' parameters: #(Oop ).


	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^self cCode: 'getInfoTypeLong(curlHandle, CURLINFO_REQUEST_SIZE )'
	! !

!CurlPlugin methodsFor: 'primitives easy-getinfo-long' stamp: 'dao 9/24/2006 18:26'!
primInfoResponseCode: aLargeIntegerCurlHandle
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primInfoResponseCode' parameters: #(Oop ).


	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^self cCode: 'getInfoTypeLong(curlHandle, CURLINFO_RESPONSE_CODE )'
	! !

!CurlPlugin methodsFor: 'primitives easy-getinfo-long' stamp: 'dao 10/1/2006 12:35'!
primInfoSSLVerifyResult: aLargeIntegerCurlHandle
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primInfoSSLVerifyResult' parameters: #(Oop ).


	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^self cCode: 'getInfoTypeLong(curlHandle, CURLINFO_SSL_VERIFYRESULT )'
	! !


!CurlPlugin methodsFor: 'primitives easy-setopt-auth' stamp: 'dao 10/7/2006 20:05'!
http: curlHandle Auth: anInteger
	

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	curlHandle easyStatusSet: ( self cCode: '(int)curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_HTTPAUTH, anInteger)').

! !

!CurlPlugin methodsFor: 'primitives easy-setopt-auth' stamp: 'dao 10/7/2006 20:13'!
primEasyHttpAuthAnySafe: aLargeIntegerCurlHandle 
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyHttpAuthAnySafe' parameters: #(Oop ).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	self cCode: 'httpAuth(curlHandle, CURLAUTH_ANYSAFE)'! !

!CurlPlugin methodsFor: 'primitives easy-setopt-auth' stamp: 'dao 10/7/2006 20:13'!
primEasyHttpAuthAny: aLargeIntegerCurlHandle 
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyHttpAuthAny' parameters: #(Oop ).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	self cCode: 'httpAuth(curlHandle, CURLAUTH_ANY)'! !

!CurlPlugin methodsFor: 'primitives easy-setopt-auth' stamp: 'dao 10/7/2006 20:13'!
primEasyHttpAuthBasic: aLargeIntegerCurlHandle 
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyHttpAuthBasic' parameters: #(Oop ).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	self cCode: 'httpAuth(curlHandle, CURLAUTH_BASIC)'! !

!CurlPlugin methodsFor: 'primitives easy-setopt-auth' stamp: 'dao 10/7/2006 20:12'!
primEasyHttpAuthDigest: aLargeIntegerCurlHandle 
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyHttpAuthDigest' parameters: #(Oop ).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	self cCode: 'httpAuth(curlHandle, CURLAUTH_DIGEST)'! !

!CurlPlugin methodsFor: 'primitives easy-setopt-auth' stamp: 'dao 10/7/2006 20:12'!
primEasyHttpAuthNtlm: aLargeIntegerCurlHandle 
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyHttpAuthNtlm' parameters: #(Oop ).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	self cCode: 'httpAuth(curlHandle, CURLAUTH_NTLM)'! !

!CurlPlugin methodsFor: 'primitives easy-setopt-auth' stamp: 'dao 10/7/2006 20:14'!
primEasyProxyAuthAnySafe: aLargeIntegerCurlHandle 
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyProxyAuthAnySafe' parameters: #(Oop ).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	self cCode: 'proxyAuth(curlHandle, CURLAUTH_ANYSAFE)'! !

!CurlPlugin methodsFor: 'primitives easy-setopt-auth' stamp: 'dao 10/7/2006 20:14'!
primEasyProxyAuthAny: aLargeIntegerCurlHandle 
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyProxyAuthAny' parameters: #(Oop ).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	self cCode: 'proxyAuth(curlHandle, CURLAUTH_ANY)'! !

!CurlPlugin methodsFor: 'primitives easy-setopt-auth' stamp: 'dao 10/7/2006 20:15'!
primEasyProxyAuthBasic: aLargeIntegerCurlHandle 
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyProxyAuthBasic' parameters: #(Oop ).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	self cCode: 'proxyAuth(curlHandle, CURLAUTH_BASIC)'! !

!CurlPlugin methodsFor: 'primitives easy-setopt-auth' stamp: 'dao 10/7/2006 20:15'!
primEasyProxyAuthDigest: aLargeIntegerCurlHandle 
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyProxyAuthDigest' parameters: #(Oop ).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	self cCode: 'proxyAuth(curlHandle, CURLAUTH_DIGEST)'! !

!CurlPlugin methodsFor: 'primitives easy-setopt-auth' stamp: 'dao 10/7/2006 20:15'!
primEasyProxyAuthNtlm: aLargeIntegerCurlHandle 
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyProxyAuthNtlm' parameters: #(Oop ).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	self cCode: 'proxyAuth(curlHandle, CURLAUTH_NTLM)'! !

!CurlPlugin methodsFor: 'primitives easy-setopt-auth' stamp: 'dao 9/24/2006 18:24'!
primEasy: aLargeIntegerCurlHandle ProxyUserPwd: aString
	

	|  curlHandle userpwdStr |
	self export: true.
	self var: #userpwdStr	declareC: 'char *userpwdStr'.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyProxyUserPwd' parameters: #(Oop Oop).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	"convert ByteString to C-string"
	userpwdStr := aString asCString.
	userpwdStr = 0 ifTrue: [^interpreterProxy primitiveFail].

	"now, set URL option, first in structure, second - in libcurl"
	curlHandle proxyUserPwdSet: userpwdStr.
	curlHandle easyStatusSet: ( self cCode: '(int)curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_PROXYUSERPWD, userpwdStr)').

! !

!CurlPlugin methodsFor: 'primitives easy-setopt-auth' stamp: 'dao 9/24/2006 18:24'!
primEasy: aLargeIntegerCurlHandle UserPwd: aString
	

	|  curlHandle userpwdStr |
	self export: true.
	self var: #userpwdStr	declareC: 'char *userpwdStr'.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyUserPwd' parameters: #(Oop Oop).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	"convert ByteString to C-string"
	userpwdStr := aString asCString.
	userpwdStr = 0 ifTrue: [^interpreterProxy primitiveFail].

	"now, set URL option, first in structure, second - in libcurl"
	curlHandle userPwdSet: userpwdStr.
	curlHandle easyStatusSet: ( self cCode: 'curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_USERPWD, userpwdStr)').

! !

!CurlPlugin methodsFor: 'primitives easy-setopt-auth' stamp: 'dao 10/7/2006 20:05'!
proxy: curlHandle Auth: anInteger
	

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	curlHandle easyStatusSet: ( self cCode: '(int)curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_PROXYAUTH, anInteger)').

! !


!CurlPlugin methodsFor: 'primitives easy-setopt-http' stamp: 'dao 10/7/2006 20:18'!
primEasy: aLargeIntegerCurlHandle AutoReferer: anInteger
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyAutoReferer' parameters: #(Oop SmallInteger).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	(anInteger = 0 or: [interpreterProxy isIntegerObject: anInteger ]) 
		ifFalse: [^interpreterProxy primitiveFail].

	"zero - be silent, 1 - be verbose"
	curlHandle easyStatusSet: ( self cCode: '(int)curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_AUTOREFERER, anInteger)').

! !

!CurlPlugin methodsFor: 'primitives easy-setopt-http' stamp: 'dao 10/8/2006 18:07'!
primEasy: aLargeIntegerCurlHandle CookieFile: aString
	

	| curlHandle cookieFileStr |
	self export: true.
	self var: #cookieFileStr	declareC: 'char *cookieFileStr'.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyCookieFile' parameters: #(Oop Oop).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	"convert ByteString to C-string"
	cookieFileStr := aString asCString.
	cookieFileStr = 0 ifTrue: [^interpreterProxy primitiveFail].

	curlHandle cookieFileSet: cookieFileStr.
	curlHandle easyStatusSet: (self cCode: '(int)curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_COOKIEFILE , cookieFileStr)').

! !

!CurlPlugin methodsFor: 'primitives easy-setopt-http' stamp: 'dao 10/8/2006 18:08'!
primEasy: aLargeIntegerCurlHandle CookieJar: aString
	

	| curlHandle cookieJarStr |
	self export: true.
	self var: #cookieJarStr	declareC: 'char *cookieJarStr'.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyCookieJar' parameters: #(Oop Oop).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	"convert ByteString to C-string"
	cookieJarStr := aString asCString.
	cookieJarStr = 0 ifTrue: [^interpreterProxy primitiveFail].

	curlHandle cookieJarSet: cookieJarStr.
	curlHandle easyStatusSet: (self cCode: '(int)curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_COOKIEJAR , cookieJarStr)').

! !

!CurlPlugin methodsFor: 'primitives easy-setopt-http' stamp: 'dao 10/8/2006 17:33'!
primEasy: aLargeIntegerCurlHandle CookieSession: anInteger
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyCookieSession' parameters: #(Oop SmallInteger).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	(anInteger = 0 or: [interpreterProxy isIntegerObject: anInteger ]) 
		ifFalse: [^interpreterProxy primitiveFail].

	curlHandle easyStatusSet: (self cCode: '(int)curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_COOKIESESSION , anInteger)').
	
! !

!CurlPlugin methodsFor: 'primitives easy-setopt-http' stamp: 'dao 10/8/2006 18:08'!
primEasy: aLargeIntegerCurlHandle Cookie: aString
	

	| curlHandle cookieStr |
	self export: true.
	self var: #cookieStr	declareC: 'char *cookieStr'.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyCookie' parameters: #(Oop Oop).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	"convert ByteString to C-string"
	cookieStr := aString asCString.
	cookieStr = 0 ifTrue: [^interpreterProxy primitiveFail].

	curlHandle cookieSet: cookieStr.
	curlHandle easyStatusSet: (self cCode: '(int)curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_COOKIE , cookieStr)').

! !

!CurlPlugin methodsFor: 'primitives easy-setopt-http' stamp: 'dao 10/1/2006 23:36'!
primEasy: aLargeIntegerCurlHandle FollowLocation: anInteger
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyFollowLocation' parameters: #(Oop SmallInteger).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	(anInteger = 0 or: [interpreterProxy isIntegerObject: anInteger ]) 
		ifFalse: [^interpreterProxy primitiveFail].

	"zero - be silent, 1 - be verbose"
	curlHandle easyStatusSet: ( self cCode: '(int)curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_FOLLOWLOCATION, anInteger)').

! !

!CurlPlugin methodsFor: 'primitives easy-setopt-http' stamp: 'dao 10/1/2006 18:37'!
primEasy: aLargeIntegerCurlHandle Header: anInteger
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyHeader' parameters: #(Oop SmallInteger).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	(anInteger = 0 or: [interpreterProxy isIntegerObject: anInteger ]) 
		ifFalse: [^interpreterProxy primitiveFail].

	"zero -return headers, 1 - strip headers"
	curlHandle easyStatusSet: (self cCode: '(int)curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_HEADER , anInteger)').
	
! !

!CurlPlugin methodsFor: 'primitives easy-setopt-http' stamp: 'dao 10/7/2006 20:26'!
primEasy: aLargeIntegerCurlHandle HttpGet: anInteger
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyHttpGet' parameters: #(Oop SmallInteger).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	(anInteger = 0 or: [interpreterProxy isIntegerObject: anInteger ]) 
		ifFalse: [^interpreterProxy primitiveFail].

	"zero -return headers, 1 - strip headers"
	curlHandle easyStatusSet: (self cCode: '(int)curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_HTTPGET , anInteger)').
	
! !

!CurlPlugin methodsFor: 'primitives easy-setopt-http' stamp: 'dao 10/8/2006 17:50'!
primEasy: aLargeIntegerCurlHandle MaxRedirs: anInteger
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyMaxRedirs' parameters: #(Oop SmallInteger).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	(anInteger = 0 or: [interpreterProxy isIntegerObject: anInteger ]) 
		ifFalse: [^interpreterProxy primitiveFail].

	curlHandle easyStatusSet: (self cCode: '(int)curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_MAXREDIRS , anInteger)').
	
! !

!CurlPlugin methodsFor: 'primitives easy-setopt-http' stamp: 'dao 10/7/2006 20:24'!
primEasy: aLargeIntegerCurlHandle NoBody: anInteger
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyNoBody' parameters: #(Oop SmallInteger).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	(anInteger = 0 or: [interpreterProxy isIntegerObject: anInteger ]) 
		ifFalse: [^interpreterProxy primitiveFail].

	"zero -return headers, 1 - strip headers"
	curlHandle easyStatusSet: (self cCode: '(int)curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_NOBODY , anInteger)').
	
! !

!CurlPlugin methodsFor: 'primitives easy-setopt-http' stamp: 'dao 10/7/2006 20:42'!
primEasy: aLargeIntegerCurlHandle Post: anInteger
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyPost' parameters: #(Oop SmallInteger).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	(anInteger = 0 or: [interpreterProxy isIntegerObject: anInteger ]) 
		ifFalse: [^interpreterProxy primitiveFail].
	
	"going to do a post - set the data size of the buffer"
	self cCode: 'curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_POSTFIELDSIZE, dataSizeGet(curlHandle))'.

	curlHandle easyStatusSet: ( self cCode: '(int)curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_POST, anInteger)').

! !

!CurlPlugin methodsFor: 'primitives easy-setopt-http' stamp: 'dao 10/1/2006 23:37'!
primEasy: aLargeIntegerCurlHandle UnrestrictedAuth: anInteger
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyUnrestrictedAuth' parameters: #(Oop SmallInteger).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	(anInteger = 0 or: [interpreterProxy isIntegerObject: anInteger ]) 
		ifFalse: [^interpreterProxy primitiveFail].

	"zero - be silent, 1 - be verbose"
	curlHandle easyStatusSet: ( self cCode: '(int)curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_UNRESTRICTED_AUTH, anInteger)').

! !


!CurlPlugin methodsFor: 'primitives easy-setopt' stamp: 'dao 9/24/2006 18:23'!
primEasy: aLargeIntegerCurlHandle CaInfo: aString
	

	| curlHandle cainfoStr |
	self export: true.
	self var: #cainfoStr	declareC: 'char *cainfoStr'.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyCaInfo' parameters: #(Oop Oop).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	"convert ByteString to C-string"
	cainfoStr := aString asCString.
	cainfoStr = 0 ifTrue: [^interpreterProxy primitiveFail].

	"now, set CAINFO option, first in structure, second - in libcurl"
	curlHandle cainfoSet: cainfoStr.
	curlHandle easyStatusSet: (self cCode: '(int)curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_CAINFO , cainfoStr)').

! !

!CurlPlugin methodsFor: 'primitives easy-setopt' stamp: 'dao 10/1/2006 18:38'!
primEasy: aLargeIntegerCurlHandle DNSCacheTimeout: anInteger
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyDNSCacheTimeout' parameters: #(Oop SmallInteger).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].


	(anInteger = 0 or: [interpreterProxy isIntegerObject: anInteger ]) 
		ifFalse: [^interpreterProxy primitiveFail].

	"zero - be not verify ssl, 1 - verify"
	curlHandle easyStatusSet: (self cCode: '(int)curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_DNS_CACHE_TIMEOUT, anInteger)').
! !

!CurlPlugin methodsFor: 'primitives easy-setopt' stamp: 'dao 10/1/2006 18:37'!
primEasy: aLargeIntegerCurlHandle FailOnError: anInteger
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyFailOnError' parameters: #(Oop SmallInteger).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	(anInteger = 0 or: [interpreterProxy isIntegerObject: anInteger ]) 
		ifFalse: [^interpreterProxy primitiveFail].

	"zero - be not verify ssl, 1 - verify"
	curlHandle easyStatusSet: (self cCode: '(int)curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_FAILONERROR, anInteger)').
! !

!CurlPlugin methodsFor: 'primitives easy-setopt' stamp: 'dao 10/1/2006 18:37'!
primEasy: aLargeIntegerCurlHandle FileTime: anInteger
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyFileTime' parameters: #(Oop SmallInteger).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	(anInteger = 0 or: [interpreterProxy isIntegerObject: anInteger ]) 
		ifFalse: [^interpreterProxy primitiveFail].

	"zero - be not verify ssl, 1 - verify"
	curlHandle easyStatusSet: (self cCode: '(int)curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_FILETIME, anInteger)').
! !

!CurlPlugin methodsFor: 'primitives easy-setopt' stamp: 'dao 10/7/2006 19:43'!
primEasy: aLargeIntegerCurlHandle HttpProxyTunnel: anInteger
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyHttpProxyTunnel' parameters: #(Oop SmallInteger).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	(anInteger = 0 or: [interpreterProxy isIntegerObject: anInteger ]) 
		ifFalse: [^interpreterProxy primitiveFail].

	"zero - be silent, 1 - be verbose"
	curlHandle easyStatusSet: ( self cCode: '(int)curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_HTTPPROXYTUNNEL, anInteger)').

! !

!CurlPlugin methodsFor: 'primitives easy-setopt' stamp: 'dao 9/24/2006 18:24'!
primEasy: aLargeIntegerCurlHandle Proxy: aString
	

	| proxyStrPtr curlHandle |
	self export: true.
	self var: #proxyStrPtr		declareC: 'char *proxyStrPtr'.
	self var: #curlHandle		declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyProxy' parameters: #(Oop Oop).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	"convert Smalltalk string to C string"
	proxyStrPtr := aString asCString.
	proxyStrPtr = 0 ifTrue: [^interpreterProxy primitiveFail].

	"now, set PROXY option, first in structure, second - in libcurl"
	curlHandle proxySet: proxyStrPtr.
	curlHandle easyStatusSet:  (self cCode: '(int)curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_PROXY, proxyStrPtr)').

! !

!CurlPlugin methodsFor: 'primitives easy-setopt' stamp: 'dao 10/1/2006 18:37'!
primEasy: aLargeIntegerCurlHandle SSLVerifyPeer: anInteger
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasySSLVerifyPeer' parameters: #(Oop SmallInteger).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	(anInteger = 0 or: [interpreterProxy isIntegerObject: anInteger ]) 
		ifFalse: [^interpreterProxy primitiveFail].

	"zero - be not verify ssl, 1 - verify"
	curlHandle easyStatusSet: (self cCode: '(int)curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_SSL_VERIFYPEER, anInteger)').
! !

!CurlPlugin methodsFor: 'primitives easy-setopt' stamp: 'dao 10/1/2006 18:37'!
primEasy: aLargeIntegerCurlHandle Timeout: anInteger
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyTimeout' parameters: #(Oop SmallInteger).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	(anInteger = 0 or: [interpreterProxy isIntegerObject: anInteger ]) 
		ifFalse: [^interpreterProxy primitiveFail].

	"zero - be not verify ssl, 1 - verify"
	curlHandle easyStatusSet: (self cCode: '(int)curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_TIMEOUT, anInteger)').
! !

!CurlPlugin methodsFor: 'primitives easy-setopt' stamp: 'dao 10/1/2006 18:36'!
primEasy: aLargeIntegerCurlHandle Upload: anInteger
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self primitive: 'primEasyUpload' parameters: #(Oop SmallInteger).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	(anInteger = 0 or: [interpreterProxy isIntegerObject: anInteger ]) 
		ifFalse: [^interpreterProxy primitiveFail].

	"going to download - set the data size of the buffer"
	self cCode: 'curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_INFILESIZE, dataSizeGet(curlHandle))'.

	"zero - donwload, 1 - upload"
	curlHandle easyStatusSet:  (self cCode: '(int)curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_UPLOAD, anInteger)').

! !

!CurlPlugin methodsFor: 'primitives easy-setopt' stamp: 'dao 9/24/2006 18:24'!
primEasy: aLargeIntegerCurlHandle Url: aString
	

	| urlStrPtr curlHandle |
	self export: true.
	self var: #urlStrPtr		declareC: 'char *urlStrPtr'.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyUrl' parameters: #(Oop Oop).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	"convert ByteString to C-string"
	urlStrPtr := aString asCString.
	urlStrPtr = 0 ifTrue: [^interpreterProxy primitiveFail].

	"now, set URL option, first in structure, second - in libcurl"
	curlHandle urlSet: urlStrPtr.
	curlHandle easyStatusSet: ( self cCode: '(int)curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_URL, urlStrPtr)').

! !

!CurlPlugin methodsFor: 'primitives easy-setopt' stamp: 'dao 10/1/2006 18:36'!
primEasy: aLargeIntegerCurlHandle Verbose: anInteger
	

	| curlHandle |
	self export: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: 'primEasyVerbose' parameters: #(Oop SmallInteger).

	"convert passed to primitive curl handle (LargeInteger) to native C-handle"
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	(anInteger = 0 or: [interpreterProxy isIntegerObject: anInteger ]) 
		ifFalse: [^interpreterProxy primitiveFail].

	"zero - be silent, 1 - be verbose"
	curlHandle easyStatusSet: ( self cCode: '(int)curl_easy_setopt( easyHandleGet(curlHandle), CURLOPT_VERBOSE, anInteger)').

! !


!CurlPlugin methodsFor: 'primitives buffers' stamp: 'dao 9/24/2006 18:23'!
primDataSize: aLargeIntegerCurlHandle 
	| curlHandle  |
	self export: true.
	self var: #curlHandle declareC: 'curlStruct* curlHandle'.
	self primitive: #primDataSize  parameters: #(#Oop ).

	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^(curlHandle dataSizeGet) asOop: Unsigned! !

!CurlPlugin methodsFor: 'primitives buffers' stamp: 'dao 9/24/2006 18:24'!
primEmptyBuffers: aLargeIntegerCurlHandle 
	

	| curlHandle |
	self export: true.
	self var: #curlHandle declareC: 'curlStruct* curlHandle'.

	self primitive: #primEmptyBuffers
		parameters: #(#Oop ).
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	curlHandle bufChainFreeAll.
! !

!CurlPlugin methodsFor: 'primitives buffers' stamp: 'dao 9/24/2006 18:24'!
primFetchBuffers: aLargeIntegerCurlHandle 

	"fetch all buffer chain contents at once, do not free buffers"
	| curlHandle curBuffer nextBuffer dataSize dataPtr byteString |
	self export: true.
	self var: #curlHandle declareC: 'curlStruct* curlHandle'.
	self var: #dataSize declareC: 'unsigned long dataSize'.
	self var: #dataPtr  declareC: 'void* dataPtr'.
	self var: #curBuffer declareC: 'dataBufferStruct* curBuffer'.
	self var: #nextBuffer declareC: 'dataBufferStruct* nextBuffer'.
	self var: #byteString declareC: 'int byteString'.
	self primitive: #primFetchBuffers
		parameters: #(#Oop ).

	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	"if there is no buffers - return nil"
	curlHandle numberOfBufGet = 0 ifTrue: [^interpreterProxy nilObject].

	"create byte array for all data in the buffer chain"
	dataSize := curlHandle dataSizeGet.
	byteString := interpreterProxy 
				instantiateClass: interpreterProxy classString
				indexableSize: dataSize.

	"set pointer to the beginning of the data area of byteString"
	dataPtr := interpreterProxy firstIndexableField: byteString.

	"copy contents of all buffers into byte array"
	curBuffer := curlHandle bufChainHeadGet.
	[curBuffer ~= 0] whileTrue: 
			["if there is at least one buffer - copy it contents into byteString"
			self cCode: 'memcpy( dataPtr, dataBufDataGet(curBuffer), dataBufSizeGet(curBuffer))'.
			"advance our data pointer"
			dataPtr := dataPtr + curBuffer dataBufSizeGet.
			"copy the pointer to the next one from it (0-pointer there is no more buffers)"
			nextBuffer := curBuffer dataBufNextGet.
			curBuffer := nextBuffer].
	^byteString! !

!CurlPlugin methodsFor: 'primitives buffers' stamp: 'dao 9/24/2006 18:24'!
primFetchNextBuffer: aLargeIntegerCurlHandle 
	| curlHandle curBuffer dataSize byteString |
"ASD"
	self export: true.
	self var: #curlHandle declareC: 'curlStruct* curlHandle'.
	self var: #dataSize declareC: 'unsigned long dataSize'.
	self var: #curBuffer declareC: 'dataBufferStruct* curBuffer'.
	self var: #byteString declareC: 'int byteString'.
	self primitive: #primFetchNextBuffer
		parameters: #(#Oop ).
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	"if there is no buffers - rewind and return nil"
	curBuffer := curlHandle readPositionBufGet.
	curBuffer = 0 ifTrue: [
		curlHandle readPositionRewind.
		^interpreterProxy nilObject].

	"create byte array of the data in the buffer"
	dataSize := curBuffer dataBufSizeGet.
	byteString := interpreterProxy 
				instantiateClass: interpreterProxy classString
				indexableSize: dataSize.

	"we copy our data from data part area of the buffer into the new byte array"
	self 
		cCode: 'memcpy( interpreterProxy->firstIndexableField(byteString), dataBufDataGet(curBuffer), dataSize )'.

	"now, we can advance the read buffer pointer"
	curlHandle readPositionBufSet: (curBuffer dataBufNextGet).
	^byteString! !

!CurlPlugin methodsFor: 'primitives buffers' stamp: 'dao 9/24/2006 18:27'!
primNumberOfBuf: aLargeIntegerCurlHandle 
	| curlHandle  |
	self export: true.
	self var: #curlHandle declareC: 'curlStruct* curlHandle'.
	self primitive: #primNumberOfBuf  parameters: #(#Oop ).

	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	^(curlHandle numberOfBufGet) asOop: Unsigned! !

!CurlPlugin methodsFor: 'primitives buffers' stamp: 'dao 9/9/2006 21:45'!
primNumberOfMemChunks

	self export: true.
	self primitive: #primNumberOfMemChunks  parameters: #().

	^ memChunks asOop: Unsigned! !

!CurlPlugin methodsFor: 'primitives buffers' stamp: 'dao 9/24/2006 18:27'!
primReadRewind: aLargeIntegerCurlHandle 
	| curlHandle  |
	self export: true.
	self var: #curlHandle declareC: 'curlStruct* curlHandle'.
	self primitive: #primReadRewind  parameters: #(#Oop ).

	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	curlHandle readPositionRewind! !

!CurlPlugin methodsFor: 'primitives buffers' stamp: 'dao 10/1/2006 18:21'!
primSet: aLargeIntegerCurlHandle BuffersData: aByteString 
	"put aByteString into a buffer chain, delete old contents"

	| curlHandle dataSize dataPtr dstPtr dstSize |
	self export: true.
	self var: #curlHandle declareC: 'curlStruct* curlHandle'.
	self var: #dataSize declareC: 'unsigned long dataSize'.
	self var: #dataPtr declareC: 'void* dataPtr'.
	self var: #dstSize declareC: 'unsigned long dstSize'.
	self var: #dstPtr declareC: 'void* dstPtr'.
	self var: #aByteString declareC: 'int aByteString'.
	self primitive: #primSetBuffersData
		parameters: #(#Oop #Oop ).
	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].

	(interpreterProxy isBytes: aByteString ) ifFalse: [^interpreterProxy primitiveFail].

	"delete all contents"
	curlHandle bufChainFreeAll.

	"get the size of an array"
	dataSize := interpreterProxy stSizeOf: aByteString.

	"set pointer to the beginning of the data area of aByteString"
	dataPtr := interpreterProxy firstIndexableField: aByteString.

	"while data is left continoulsy allocate buffers and fill them with data"
	[dataSize > 0] whileTrue: 
			["get size we need"

			dstSize := dataSize min: self curlMaxWriteSize.
			"allocate one buffer"
			dstPtr := curlHandle bufChainAddBuffer: dstSize.
			dstPtr = 0 
				ifTrue: 
					["problems with memory - free already grabbed and primfail"

					curlHandle bufChainFreeAll.
					^interpreterProxy primitiveFail].
			"copy one chunk"
			self cCode: 'memcpy( dstPtr, dataPtr, dstSize)'.
			
			"advance pointers and counters"
			curlHandle dataSizeSet: (dstSize + curlHandle dataSizeGet).
			dataPtr := dataPtr + dstSize.
			dataSize := dataSize - dstSize]! !


!CurlPlugin methodsFor: 'primitives handle-management' stamp: 'dao 10/8/2006 17:57'!
curlHandleFree: curlHandle

	| prev next |
	self inline: 'true'.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #prev			declareC: 'curlStruct* prev'.
	self var: #next			declareC: 'curlStruct* next'.

	prev := curlHandle prevHandleGet.
	next  := curlHandle nextHandleGet.
	prev ~= 0 
		ifTrue: [ prev nextHandleSet: next ]
		ifFalse: [ curlHandleFirst := next].
	next = 0 
		ifTrue: [curlHandleLast := prev]
		ifFalse: [next prevHandleSet: prev].

	curlHandle
			bufChainFreeAll;
			easyHandleFree;
			multiHandleFree;
			proxyUserPwdFree;
			userPwdFree;
			proxyFree;
			cainfoFree;
			cookieFree;
			cookieFileFree;
			cookieJarFree;
			urlFree;
			cFree
	
	
	
	
	! !

!CurlPlugin methodsFor: 'primitives handle-management' stamp: 'dao 9/24/2006 20:26'!
curlHandleNew

	| curlHandle easyHandle multiHandle |
	self export: 'true'.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #easyHandle	declareC: 'CURL* easyHandle'.
	self var: #multiHandle	declareC: 'CURLM* multiHandle'.
	
	self returnTypeC: 'curlStruct *'.


	"allocate memory for the handle structure"

	curlHandle := self sizeOfCurlStruct cMalloc.
	curlHandle = 0 ifTrue: [^0].

	"wipe out fields"
	self cCode: 'memset(curlHandle, 0, sizeOfCurlStruct())'.

	"request 'easy' handle from curllib"
	easyHandle := self cCode: 'curl_easy_init()'.
	easyHandle = 0 ifTrue: [
		curlHandle cFree.
		^0].

	"store it in our structure"
	curlHandle easyHandleSet: easyHandle.

	multiHandle :=  self cCode: 'curl_multi_init()'.
	multiHandle = 0 ifTrue: [
		curlHandle easyHandleFree.
		curlHandle cFree.
		^0].
	curlHandle multiHandleSet: multiHandle.

	"register default write and read function callbacks"	
	
	self cCode: 'curl_easy_setopt( easyHandle, CURLOPT_WRITEFUNCTION, pvtWriteNewBufferItemsofBytesUserData)'.
	self cCode: 'curl_easy_setopt( easyHandle, CURLOPT_WRITEDATA, curlHandle)'.
	self cCode: 'curl_easy_setopt( easyHandle, CURLOPT_READFUNCTION, pvtReadBufferItemsofBytesUserData)'.
	self cCode: 'curl_easy_setopt( easyHandle, CURLOPT_READDATA, curlHandle)'.
	
	"disable progress-meter just in case"
	self cCode: 'curl_easy_setopt( easyHandle, CURLOPT_NOPROGRESS, 1)'.
	self cCode: 'curl_easy_setopt( easyHandle, CURLOPT_SHARE, curlShare)'.

	curlHandle prevHandleSet: curlHandleLast.
	curlHandle nextHandleSet: 0.
	curlHandleFirst = 0 ifTrue: [curlHandleFirst := curlHandle].
	curlHandleLast ~= 0 ifTrue: [curlHandleLast nextHandleSet: curlHandle].
	curlHandleLast := curlHandle.

	^curlHandle 
	
	
	! !

!CurlPlugin methodsFor: 'primitives handle-management' stamp: 'dao 9/24/2006 17:44'!
freeAllCurlHandles

	| headHandle nextHandle  |
	self inline: true.
	self var: #headHandle declareC: 'curlStruct* headHandle'.
	self var: #nextHandle declareC: 'curlStruct* nextHandle'.

	"release all buffers"
	headHandle := curlHandleFirst.
	[headHandle ~= 0] whileTrue: 
			["if there is at least one handle"

			"copy the pointer to the next one from it (0-pointer there is no more handles)"

			nextHandle := headHandle nextHandleGet.
			headHandle curlHandleFree.
			headHandle := nextHandle].

	curlHandleFirst := 0.
	curlHandleLast := 0.
	! !

!CurlPlugin methodsFor: 'primitives handle-management' stamp: 'dao 9/24/2006 20:09'!
primFreeCurlHandle: aLargeIntegerCurlHandle

	| curlHandle |
	self export: 'true'.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: #primFreeCurlHandle parameters: #(Oop).

	curlHandle := aLargeIntegerCurlHandle asCurlHandle.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	curlHandle curlHandleFree.
	
	
	! !

!CurlPlugin methodsFor: 'primitives handle-management' stamp: 'dao 9/24/2006 18:07'!
primNewCurlHandle

	| curlHandle |
	self export: 'true'.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self primitive: #primNewCurlHandle parameters: #().

	curlHandle := self curlHandleNew.
	curlHandle = 0 ifTrue: [^interpreterProxy primitiveFail].
	^curlHandle asLargeIntegerCurlHandle
	
	
	! !


!CurlPlugin methodsFor: 'pvt-c converting' stamp: 'dao 10/1/2006 18:06'!
asCString: aString
	
	| strPtr strSize |
	self inline: true.
	self var: #aString		declareC: 'int aString'.
	self var: #strPtr		declareC: 'char *strPtr'.
	self var: #strSize		declareC: 'int strSize'.
	self returnTypeC: 'char *'.

	(interpreterProxy isBytes: aString) ifFalse: [^0].

	strSize := (interpreterProxy stSizeOf: aString) + 1.
	strPtr := strSize cMalloc.
	strPtr = 0 ifTrue: [^0].

	"copy chars from the ByteString to allocated place, put 0 at end"
	self cCode: 'strncpy( strPtr, interpreterProxy->firstIndexableField(aString), strSize)'.
	strPtr at: strSize-1 put: 0.

	^strPtr! !

!CurlPlugin methodsFor: 'pvt-c converting' stamp: 'dao 9/24/2006 18:20'!
asCurlHandle: aLargeIntegerCurlHandle
	
	"convert Smalltalk side LargeInteger representation of the pointer to the C pointer"
	| curlHandle current |
	self inline: true.
	self var: #aLargeIntegerCurlHandle declareC: 'int aLargeIntegerCurlHandle'.
	self var: #curlHandle declareC: 'curlStruct * curlHandle'.
	self var: #current declareC: 'curlStruct * current'.

	self returnTypeC: 'curlStruct*'.

	curlHandle := self cCoerce: (interpreterProxy positive32BitValueOf: aLargeIntegerCurlHandle) to: 'curlStruct*'.

	current := curlHandleFirst.
	[current ~= 0] whileTrue:
		[current = curlHandle
			ifTrue: [^curlHandle]
			ifFalse: [current := current nextHandleGet]].
	^0! !

!CurlPlugin methodsFor: 'pvt-c converting' stamp: 'dao 9/16/2006 13:40'!
asLargeIntegerCurlHandle: curlHandle
	
	"convert C-curlHandle to the Smalltalk LargeInteger "
	self inline: true.
	self var: #curlHandle declareC: 'curlStruct *curlHandle'.
	self returnTypeC: 'int'.

	^(self cCoerce: curlHandle to: 'int') asOop: Unsigned! !

!CurlPlugin methodsFor: 'pvt-c converting' stamp: 'dao 10/1/2006 15:22'!
asString: strPtr
	
	| strSize aString |
	self inline: 'true'.
	self var: #strPtr		declareC: 'char* strPtr'.
	self var: #strSize		declareC: 'int strSize'.
	self var: #aString		declareC: 'int aString'.

	strPtr = 0 
		ifTrue: [strSize := 0 ]
		ifFalse: [strSize := strPtr strlen].
	aString := interpreterProxy instantiateClass: interpreterProxy classString indexableSize: strSize.
	strSize > 0 ifTrue: [self cCode: 'strncpy( interpreterProxy->firstIndexableField(aString), strPtr, strSize )'].
	^aString

! !


!CurlPlugin methodsFor: 'pvt-c memory' stamp: 'dao 9/16/2006 13:25'!
cFree: ptr

	self inline: true.
	self returnTypeC: 'void'.
	self var: #ptr	declareC: 'void* ptr'.
	
	ptr ~= 0 ifTrue: [ memChunks := memChunks - 1 ].
	ptr free


	! !

!CurlPlugin methodsFor: 'pvt-c memory' stamp: 'dao 9/16/2006 13:25'!
cMalloc: sizeInt

	| ptr |
	self inline: true.
	self returnTypeC: 'void*'.
	self var: #sizeInt declareC: 'int sizeInt'.
	self var: #ptr	declareC: 'void* ptr'.
	
	ptr := sizeInt malloc.
	ptr ~= 0 ifTrue: [ memChunks := memChunks + 1 ].
	^ptr

	! !


!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/18/2006 16:22'!
cainfoFree: curlHandle

	| cainfo |
	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #cainfo		declareC: 'char* cainfo'.
	
	cainfo := self cCode: 'curlHandle->cainfo'.
	cainfo ~= 0 ifTrue: [cainfo cFree ].
	self cCode: 'curlHandle->cainfo = 0'.! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/18/2006 16:16'!
cainfoGet: curlHandle

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self returnTypeC: 'char *'.
	
	^self cCode: 'curlHandle->cainfo'.
! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/18/2006 16:16'!
cainfo: curlHandle Set: str

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #str			declareC: 'char*  str'.

	curlHandle cainfoFree.
	self cCode: 'curlHandle->cainfo=str'.

	
	
	
	
	! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 10/8/2006 17:55'!
cookieFileFree: curlHandle

	| cookieFile |
	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #cookieFile		declareC: 'char* cookieFile'.
	
	cookieFile := self cCode: 'curlHandle->cookieFile'.
	cookieFile ~= 0 ifTrue: [cookieFile cFree ].
	self cCode: 'curlHandle->cookieFile = 0'.! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 10/8/2006 17:55'!
cookieFileGet: curlHandle

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self returnTypeC: 'char *'.
	
	^self cCode: 'curlHandle->cookieFile'.
! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 10/8/2006 17:56'!
cookieFile: curlHandle Set: str

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #str			declareC: 'char*  str'.

	curlHandle cookieFileFree.
	self cCode: 'curlHandle->cookieFile=str'.

	
	
	
	
	! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 10/8/2006 17:54'!
cookieFree: curlHandle

	| cookie |
	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #cookie		declareC: 'char* cookie'.
	
	cookie := self cCode: 'curlHandle->cookie'.
	cookie ~= 0 ifTrue: [cookie cFree ].
	self cCode: 'curlHandle->cookie = 0'.! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 10/8/2006 17:55'!
cookieGet: curlHandle

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self returnTypeC: 'char *'.
	
	^self cCode: 'curlHandle->cookie'.
! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 10/8/2006 17:55'!
cookieJarFree: curlHandle

	| cookieJar |
	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #cookieJar		declareC: 'char* cookieJar'.
	
	cookieJar := self cCode: 'curlHandle->cookieJar'.
	cookieJar ~= 0 ifTrue: [cookieJar cFree ].
	self cCode: 'curlHandle->cookieJar = 0'.! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 10/8/2006 17:55'!
cookieJarGet: curlHandle

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self returnTypeC: 'char *'.
	
	^self cCode: 'curlHandle->cookieJar'.
! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 10/8/2006 17:56'!
cookieJar: curlHandle Set: str

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #str			declareC: 'char*  str'.

	curlHandle cookieJarFree.
	self cCode: 'curlHandle->cookieJar=str'.

	
	
	
	
	! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 10/8/2006 17:56'!
cookie: curlHandle Set: str

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #str			declareC: 'char*  str'.

	curlHandle cookieFree.
	self cCode: 'curlHandle->cookie=str'.

	
	
	
	
	! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/18/2006 16:17'!
easyHandleFree: curlHandle 

	| easyHandle |
	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #easyHandle	declareC: 'CURL* easyHandle'.

	easyHandle := self cCode: 'curlHandle->easyHandle'.
	easyHandle ~= 0 ifTrue: [
		self cCode: 'curl_easy_setopt( easyHandle, CURLOPT_SHARE, 0)'.
		self cCode: 'curl_easy_cleanup(easyHandle)'].
	self cCode: 'curlHandle->easyHandle = 0'.

	
	
	
	
	! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/18/2006 16:17'!
easyHandleGet: curlHandle

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self returnTypeC: 'CURL *'.	

	^self cCode: 'curlHandle->easyHandle'.

	
	
	
	
	! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/18/2006 16:17'!
easyHandle: curlHandle Set: easyHandle

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #easyHandle	declareC: 'CURL* easyHandle'.

	curlHandle easyHandleFree.
	self cCode: 'curlHandle->easyHandle = easyHandle'.

	
	
	
	
	! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/18/2006 16:17'!
easyStatusGet: curlHandle

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self returnTypeC: 'CURLcode'.
	
	^self cCode: 'curlHandle->easyStatus'.
! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/18/2006 16:17'!
easyStatus: curlHandle Set: status

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #status			declareC: 'CURLcode status'.


	self cCode: 'curlHandle->easyStatus = status'.

	
	
	
	
	! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/18/2006 16:17'!
isMultiRunningGet: curlHandle

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self returnTypeC: 'int '.
	
	^self cCode: 'curlHandle->isMultiRunning'.
! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/18/2006 16:17'!
isMultiRunningPtr: curlHandle

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self returnTypeC: 'int *'.
	
	^self cCode: '&curlHandle->isMultiRunning'.
! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/18/2006 16:21'!
is: curlHandle MultiRunningSet: running

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #running	declareC: 'int running'.


	self cCode: 'curlHandle->isMultiRunning = running'.

	
	
	
	
	! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/16/2006 19:51'!
multiHandleFree: curlHandle 

	| multiHandle |
	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #multiHandle	declareC: 'CURLM* multiHandle'.

	multiHandle := self cCode: 'curlHandle->multiHandle'.
	multiHandle ~= 0 ifTrue: [
		self cCode: 'curl_multi_cleanup(multiHandle)'].
	self cCode: 'curlHandle->multiHandle = 0'.

	
	
	
	
	! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/16/2006 18:44'!
multiHandleGet: curlHandle

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self returnTypeC: 'CURLM *'.	

	^self cCode: 'curlHandle->multiHandle'.

	
	
	
	
	! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/16/2006 18:44'!
multiHandle: curlHandle Set: multiHandle

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #multiHandle	declareC: 'CURLM* multiHandle'.

	curlHandle multiHandleFree.
	self cCode: 'curlHandle->multiHandle = multiHandle'.

	
	
	
	
	! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/18/2006 16:17'!
multiStatusGet: curlHandle

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self returnTypeC: 'CURLMcode'.
	
	^self cCode: 'curlHandle->multiStatus'.
! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/18/2006 16:17'!
multiStatus: curlHandle Set: status

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #status			declareC: 'CURLMcode status'.


	self cCode: 'curlHandle->multiStatus = status'.

	
	
	
	
	! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/24/2006 17:47'!
nextHandleGet: curlHandle

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self returnTypeC: 'curlStruct *'.	

	^self cCode: '(curlStruct *) curlHandle->next'.

	
	
	
	
	! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/24/2006 17:50'!
next: curlHandle HandleSet: nextHandle

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #nextHandle	declareC: 'curlStruct* nextHandle'.

	self cCode: 'curlHandle->next = (struct curlStruct *)nextHandle'.

	
	
	
	
	! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/24/2006 17:47'!
prevHandleGet: curlHandle

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self returnTypeC: 'curlStruct *'.	

	^self cCode: '(curlStruct *)curlHandle->prev'.

	
	
	
	
	! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/24/2006 17:50'!
prev: curlHandle HandleSet: prevHandle

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #prevHandle	declareC: 'curlStruct* prevHandle'.

	self cCode: 'curlHandle->prev = (struct curlStruct *)prevHandle'.

	
	
	
	
	! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/18/2006 16:18'!
proxyFree: curlHandle

	| proxy |
	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #proxy		declareC: 'char* proxy'.
	
	proxy := self cCode: 'curlHandle->proxy'.
	proxy ~= 0 ifTrue: [proxy cFree ].
	self cCode: 'curlHandle->proxy = 0'.! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/18/2006 16:18'!
proxyGet: curlHandle

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self returnTypeC: 'char *'.
	
	^self cCode: 'curlHandle->proxy'.
! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/16/2006 17:02'!
proxyUserPwdFree: curlHandle

	| proxyUserPwd |
	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #proxyUserPwd		declareC: 'char* proxyUserPwd'.
	
	proxyUserPwd := self cCode: 'curlHandle->proxyUserPwd'.
	proxyUserPwd ~= 0 ifTrue: [proxyUserPwd cFree ].
	self cCode: 'curlHandle->proxyUserPwd = 0'.! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/16/2006 16:33'!
proxyUserPwdGet: curlHandle

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self returnTypeC: 'char *'.
	
	^self cCode: 'curlHandle->proxyUserPwd'.
! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/18/2006 16:18'!
proxy: curlHandle Set: str

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #str			declareC: 'char*  str'.

	curlHandle proxyFree.
	self cCode: 'curlHandle->proxy = str'.

	
	
	
	
	! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/16/2006 16:34'!
proxy: curlHandle UserPwdSet: str

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #str			declareC: 'char*  str'.

	curlHandle proxyFree.
	self cCode: 'curlHandle->proxyUserPwd = str'.

	
	
	
	
	! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/16/2006 10:44'!
sizeOfCurlStruct

	self inline: true.
	^self cCode: 'sizeof(curlStruct)'! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/18/2006 16:18'!
urlFree: curlHandle

	| url |

	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #url			declareC: 'char* url'.
	
	url := self cCode: 'curlHandle->url'.
	url ~= 0 ifTrue: [url cFree ].
	self cCode: 'curlHandle->url = 0'! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/18/2006 16:18'!
urlGet: curlHandle

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self returnTypeC: 'char *'.
	
	^self cCode: 'curlHandle->url'
! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/18/2006 16:18'!
url: curlHandle Set: str


	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #str			declareC: 'char*  str'.

	curlHandle urlFree.
	self cCode: 'curlHandle->url = str'

	
	
	
	
	! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/16/2006 17:02'!
userPwdFree: curlHandle

	| userPwd |
	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #userPwd		declareC: 'char* userPwd'.
	
	userPwd := self cCode: 'curlHandle->userPwd'.
	userPwd ~= 0 ifTrue: [ userPwd cFree].
	self cCode: 'curlHandle->userPwd = 0'.! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/16/2006 16:55'!
userPwdGet: curlHandle

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self returnTypeC: 'char *'.
	
	^self cCode: 'curlHandle->userPwd'.! !

!CurlPlugin methodsFor: 'pvt-c handle-mgmt' stamp: 'dao 9/16/2006 16:56'!
userPwd: curlHandle Set: str

	self inline: true.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #str			declareC: 'char*  str'.

	curlHandle userPwdFree.
	self cCode: 'curlHandle->userPwd = str'.

	
	
	
	
	! !


!CurlPlugin methodsFor: 'pvt-c data-buffer' stamp: 'dao 9/7/2006 18:49'!
curlMaxWriteSize
	"the default buffer size used by curl library - use it when not sure about size to allocate memory
	 this is a constant defined in curl.h "
	self inline: true.
	^self cCode: 'CURL_MAX_WRITE_SIZE'

	! !

!CurlPlugin methodsFor: 'pvt-c data-buffer' stamp: 'dao 9/16/2006 12:48'!
dataBufDataGet: dataBuffer 
	self inline: true.
	self var: #dataBuffer declareC: 'dataBufferStruct* dataBuffer'.
	self returnTypeC: 'void *'.

	^self cCode: '&dataBuffer->data'.

! !

!CurlPlugin methodsFor: 'pvt-c data-buffer' stamp: 'dao 9/16/2006 14:04'!
dataBufNextGet: dataBuffer
	self inline: true.
	self var: #dataBuffer declareC: 'dataBufferStruct* dataBuffer'.
	self returnTypeC: 'void*'.

	^self cCode: 'dataBuffer->next'.! !

!CurlPlugin methodsFor: 'pvt-c data-buffer' stamp: 'dao 9/16/2006 14:04'!
dataBufPrevGet: dataBuffer 
	self inline: true.
	self var: #dataBuffer declareC: 'dataBufferStruct* dataBuffer'.
	self returnTypeC: 'void*'.

	^self cCode: 'dataBuffer->prev'.! !

!CurlPlugin methodsFor: 'pvt-c data-buffer' stamp: 'dao 9/16/2006 12:40'!
dataBufSizeGet: dataBuffer

	self inline: true.
	self var: #dataBuffer declareC: 'dataBufferStruct* dataBuffer'.
	self returnTypeC: 'unsigned long '.

	^self cCode: 'dataBuffer->size'.

	! !

!CurlPlugin methodsFor: 'pvt-c data-buffer' stamp: 'dao 9/16/2006 14:39'!
dataBuf: dataBuffer NextSet: nextDataBuffer

	self inline: true.
	self var: #dataBuffer declareC: 'dataBufferStruct* dataBuffer'.
	self var: #nextDataBuffer declareC: 'void* nextDataBuffer'.

	self cCode: 'dataBuffer->next = nextDataBuffer'.
	

	! !

!CurlPlugin methodsFor: 'pvt-c data-buffer' stamp: 'dao 9/16/2006 14:05'!
dataBuf: dataBuffer PrevSet: prevDataBuffer

	self inline: true.
	self var: #dataBuffer 		declareC: 'dataBufferStruct* dataBuffer'.
	self var: #prevDataBuffer 	declareC: 'void* prevDataBuffer'.

	self cCode: 'dataBuffer->prev = prevDataBuffer'.
	! !

!CurlPlugin methodsFor: 'pvt-c data-buffer' stamp: 'dao 9/16/2006 12:44'!
dataBuf: dataBuffer SizeSet: size

	self inline: true.
	self var: #dataBuffer 	declareC: 'dataBufferStruct* dataBuffer'.

	self cCode: 'dataBuffer->size = size'.

	! !


!CurlPlugin methodsFor: 'pvt-c buffer-position' stamp: 'dao 9/16/2006 12:03'!
adjust: curlHandle ReadPositionForHeadBufferDeletion: dataBufferStruct
	self inline: true.
	self var: #curlHandle declareC: 'curlStruct* curlHandle'.
	self var: #dataBufferStruct declareC: 'dataBufferStruct* dataBufferStruct'.


	(dataBufferStruct dataBufNextGet = 0) ifTrue: [
		"buffer being deleted is the last - set all pos to zero"
			curlHandle readPositionRewind ]	

		ifFalse: [
			"move all pointers to the beggining of the next buffer if they currently in the being deleted one"

			curlHandle readPositionBufGet = dataBufferStruct ifTrue: [
				curlHandle readPositionBufSet: (dataBufferStruct dataBufNextGet ).
				curlHandle readPositionSet: 0 ]].! !

!CurlPlugin methodsFor: 'pvt-c buffer-position' stamp: 'dao 9/18/2006 16:18'!
readPositionBufGet: curlHandle 
	self inline: true.
	self var: #curlHandle declareC: 'curlStruct* curlHandle'.
	self returnTypeC: 'void*'.

	^self cCode: 'curlHandle->readPositionBuf'.! !

!CurlPlugin methodsFor: 'pvt-c buffer-position' stamp: 'dao 9/18/2006 16:18'!
readPositionGet: curlHandle 
	self inline: true.
	self var: #curlHandle declareC: 'curlStruct* curlHandle'.
	self returnTypeC: 'unsigned'.

	^self cCode: 'curlHandle->readPosition'.! !

!CurlPlugin methodsFor: 'pvt-c buffer-position' stamp: 'dao 9/16/2006 14:00'!
readPositionIsAtEnd: curlHandle 
	| currentBuf |
	self inline: true.
	self var: #curlHandle declareC: 'curlStruct* curlHandle'.
	self var: #currentBuf declareC: 'dataBufferStruct* currentBuf'.
	self returnTypeC: 'int'.

	currentBuf := curlHandle readPositionBufGet.
	currentBuf = 0 ifTrue: [
		"no buffers yet - we are at end" ^1].
	((currentBuf = 0) and: [(curlHandle readPositionGet = currentBuf dataBufSizeGet)])
		ifTrue: [^1 "we are in the last buffer (no next) and the position is at the very end of it"]
		ifFalse: [^0]! !

!CurlPlugin methodsFor: 'pvt-c buffer-position' stamp: 'dao 9/16/2006 12:01'!
readPositionRewind: curlHandle 
	self inline: true.
	self var: #curlHandle declareC: 'curlStruct* curlHandle'.

	curlHandle
		readPositionBufSet: (curlHandle bufChainHeadGet); 
		readPositionSet: 0! !

!CurlPlugin methodsFor: 'pvt-c buffer-position' stamp: 'dao 9/18/2006 16:18'!
readPosition: curlHandle BufSet: dataBufferStruct 
	self inline: true.
	self var: #curlHandle declareC: 'curlStruct* curlHandle'.
	self var: #dataBufferStruct declareC: 'dataBufferStruct* dataBufferStruct'.
	
	self cCode: 'curlHandle->readPositionBuf = dataBufferStruct'.! !

!CurlPlugin methodsFor: 'pvt-c buffer-position' stamp: 'dao 9/18/2006 16:18'!
readPosition: curlHandle Set: pos 
	self inline: true.
	self var: #curlHandle declareC: 'curlStruct* curlHandle'.
	self var: #pos declareC: 'unsigned pos'.

	self cCode: 'curlHandle->readPosition = pos'.! !


!CurlPlugin methodsFor: 'pvt-c buffers-chain' stamp: 'dao 9/16/2006 12:04'!
bufChainFreeAll: curlHandle 
	| headBuffer nextBuffer |
	self inline: true.
	self var: #curlHandle declareC: 'curlStruct* curlHandle'.
	self var: #headBuffer declareC: 'dataBufferStruct* headBuffer'.
	self var: #nextBuffer declareC: 'dataBufferStruct* nextBuffer'.

	"release all buffers"
	headBuffer := curlHandle bufChainHeadGet.
	[headBuffer ~= 0] whileTrue: 
			["if there is at least one buffer"

			"copy the pointer to the next one from it (0-pointer there is no more buffers)"

			nextBuffer := headBuffer dataBufNextGet.
			headBuffer cFree.
			headBuffer := nextBuffer].
	curlHandle
		bufChainHeadSet: 0;
		bufChainTailSet: 0;
		dataSizeSet:	0; numberOfBufSet: 0;
		readPositionRewind.! !

!CurlPlugin methodsFor: 'pvt-c buffers-chain' stamp: 'dao 9/17/2006 15:49'!
bufChainFreeHeadBuffer: curlHandle 
	| headBuffer nextBuffer |
	self inline: true.
	self var: #curlHandle declareC: 'curlStruct* curlHandle'.
	self var: #headBuffer declareC: 'dataBufferStruct* headBuffer'.
	self var: #nextBuffer declareC: 'dataBufferStruct* nextBuffer'.

	"get the head buffer in chain"
	headBuffer := curlHandle bufChainHeadGet.
	headBuffer ~= 0 
		ifTrue: 
			["if there is at least one buffer"

			"copy the pointer to the next one from it (0-pointer there is no more buffers)"

			nextBuffer := headBuffer dataBufNextGet.
			curlHandle bufChainHeadSet: nextBuffer.	"store that pointer as the first in chain"
			nextBuffer = 0 
				ifTrue: 
					[curlHandle bufChainTailSet: 0	"when no buffers left head == tail == 0"]
				ifFalse: 
					[" next buffer actually exists"

					nextBuffer dataBufNextGet = 0 
						ifTrue: 
							["if the next buffer happened to be the last"

							curlHandle bufChainTailSet: nextBuffer	"store it as a tail"]]].

	"decrement number of buffers and bytes left"
	curlHandle numberOfBufSet: (curlHandle numberOfBufGet - 1).
	curlHandle dataSizeSet: (curlHandle dataSizeGet - headBuffer dataBufSizeGet ).
	curlHandle adjustReadPositionForHeadBufferDeletion: headBuffer.

	"now we can free memory"
	headBuffer cFree! !

!CurlPlugin methodsFor: 'pvt-c buffers-chain' stamp: 'dao 10/1/2006 23:17'!
bufChainHeadGet: curlHandle 
	self inline: true.
	self var: #curlHandle declareC: 'curlStruct* curlHandle'.
	self returnTypeC: 'void*'.

	^self cCode: 'curlHandle->dataBufHead'.! !

!CurlPlugin methodsFor: 'pvt-c buffers-chain' stamp: 'dao 9/18/2006 16:16'!
bufChainHead: curlHandle Set: dataBufferStruct 
	self inline: true.
	self var: #curlHandle declareC: 'curlStruct* curlHandle'.
	self var: #dataBufferStruct declareC: 'dataBufferStruct* dataBufferStruct'.

	self cCode: 'curlHandle->dataBufHead = dataBufferStruct'.! !

!CurlPlugin methodsFor: 'pvt-c buffers-chain' stamp: 'dao 9/18/2006 16:16'!
bufChainTailGet: curlHandle 
	self inline: true.
	self var: #curlHandle declareC: 'curlStruct* curlHandle'.
	self returnTypeC: 'void*'.	

	^self cCode: 'curlHandle->dataBufTail'.! !

!CurlPlugin methodsFor: 'pvt-c buffers-chain' stamp: 'dao 9/18/2006 16:16'!
bufChainTail: curlHandle Set: dataBufferStruct 
	self inline: true.
	self var: #curlHandle declareC: 'curlStruct* curlHandle'.
	self var: #dataBufferStruct declareC: 'dataBufferStruct* dataBufferStruct'.	

	self cCode: 'curlHandle->dataBufTail = dataBufferStruct'.! !

!CurlPlugin methodsFor: 'pvt-c buffers-chain' stamp: 'dao 9/16/2006 12:04'!
bufChain: curlHandle AddBuffer: byteSize 
	| newBuffer prevBuffer |
	self inline: true.
	self var: #curlHandle declareC: 'curlStruct* curlHandle'.
	self var: #byteSize declareC: 'int byteSize'.
	self var: #newBuffer declareC: 'dataBufferStruct* newBuffer'.
	self var: #prevBuffer declareC: 'dataBufferStruct* prevBuffer'.
	self returnTypeC: 'void* '.

	"allocate memory of requested size + dataBuffer header size fields"
	newBuffer := (byteSize + self sizeOfDataBufferStruct ) cMalloc.
	newBuffer = 0 ifTrue: [^0].

	"store size in the beginning of the buffer"
	newBuffer dataBufSizeSet: byteSize.

	"increment counter of allocated buffers"
	curlHandle numberOfBufSet: (1 + curlHandle numberOfBufGet).

	"pointer to the next buffer is in the second field. New buffer is always last - so put 0 there."
	newBuffer dataBufNextSet: 0.

	
	curlHandle bufChainHeadGet = 0 
		ifTrue: "if this buffer is the first allocated one - put it in the head"
			[curlHandle bufChainHeadSet: newBuffer.
			 curlHandle readPositionBufSet: newBuffer.
			 newBuffer dataBufPrevSet: 0]
		ifFalse: "if not - we need to update the link in the previous buffer to point to this one"
			[prevBuffer := curlHandle bufChainTailGet.
			prevBuffer dataBufNextSet: newBuffer].

	"now we can update pointer to the tail"
	curlHandle bufChainTailSet: newBuffer.
	"return pointer to the data area of the new buffer"
	^newBuffer dataBufDataGet! !

!CurlPlugin methodsFor: 'pvt-c buffers-chain' stamp: 'dao 9/18/2006 16:16'!
dataSizeGet: curlHandle 
	self inline: true.
	self var: #curlHandle declareC: 'curlStruct* curlHandle'.
	self returnTypeC: 'unsigned long'.

	^self cCode: 'curlHandle->dataSize'.! !

!CurlPlugin methodsFor: 'pvt-c buffers-chain' stamp: 'dao 9/18/2006 16:16'!
data: curlHandle SizeSet: size 
	self inline: true.
	self var: #curlHandle declareC: 'curlStruct* curlHandle'.
	self var: #size declareC: 'unsigned long size'.

	self cCode: 'curlHandle->dataSize = size'.! !

!CurlPlugin methodsFor: 'pvt-c buffers-chain' stamp: 'dao 9/18/2006 16:17'!
numberOfBufGet: curlHandle 
	self inline: true.
	self var: #curlHandle declareC: 'curlStruct* curlHandle'.
	self returnTypeC: 'unsigned'.

	^self cCode: 'curlHandle->numberOfBuf'.! !

!CurlPlugin methodsFor: 'pvt-c buffers-chain' stamp: 'dao 9/18/2006 16:17'!
numberOfBuf: curlHandle Set: size 
	self inline: true.
	self var: #curlHandle declareC: 'curlStruct* curlHandle'.
	self var: #size declareC: 'unsigned size'.

	self cCode: 'curlHandle->numberOfBuf = size'.! !

!CurlPlugin methodsFor: 'pvt-c buffers-chain' stamp: 'dao 9/16/2006 11:47'!
sizeOfDataBufferStruct

	self inline: true.
	^self cCode: 'sizeof(dataBufferStruct)'! !


!CurlPlugin methodsFor: 'pvt-c buffers-callbacks' stamp: 'dao 9/16/2006 12:10'!
pvtReadBuffer: dataPtr Items: items ofBytes: size UserData: curlHandle

	"this function iterates through buffer chain and populates dataPtr with data found in buffer"
	| dataSize bytesRead unreadBytesInBuf chunkSize curBuf nextBuf   |
	self var: #dataPtr		declareC: 'void *dataPtr'.
	self var: #items			declareC: 'unsigned items'.
	self var: #size			declareC: 'unsigned size'.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.

	self var: #dataSize		declareC: 'unsigned dataSize'.
	self var: #bytesRead	declareC: 'unsigned bytesRead'.
	self var: #unreadBytesInBuf	declareC: 'int unreadBytesInBuf'.
	self var: #chunkSize	declareC: 'unsigned chunkSize'.
	self var: #curBuf			declareC: 'void* curBuf'.
	self var: #nextBuf		declareC: 'void* nextBuf'.

	"if there is nothing to read for libcurl - rewind and return 0"
	curlHandle readPositionIsAtEnd ifTrue: [curlHandle readPositionRewind. ^0]. 

	dataSize := items * size.
	bytesRead := 0.

	" cycle until buffers exhausted or read request satisfied"
	[1=1] whileTrue:
	[ curBuf := curlHandle readPositionBufGet. 
	unreadBytesInBuf := curBuf dataBufSizeGet - curlHandle readPositionGet.
	chunkSize := unreadBytesInBuf min: dataSize.

	self cCode:  'memcpy( dataPtr+bytesRead, (char*)dataBufDataGet(curBuf)+ readPositionGet(curlHandle), chunkSize )'.
	bytesRead := bytesRead + chunkSize.
	curlHandle readPositionSet: (chunkSize + curlHandle readPositionGet).

	(unreadBytesInBuf > dataSize)
		ifTrue: [ "this sub-buffer has data left in it - no need to advance to the next buffer.
				return number of data read already" ^bytesRead]

		ifFalse: [ "sub-buffer has not satisfied read request - try to move to next"
				
				(nextBuf := curBuf dataBufNextGet) = 0 
					ifTrue: ["oops there is no next buffer - return number of data read"
						^bytesRead].
				 dataSize := dataSize - chunkSize.
				 curlHandle readPositionSet: 0. 
				 curlHandle readPositionBufSet: nextBuf]] 

	! !

!CurlPlugin methodsFor: 'pvt-c buffers-callbacks' stamp: 'dao 9/16/2006 12:11'!
pvtWriteNewBuffer: dataPtr Items: items ofBytes: size UserData: curlHandle

	"this version allocates new buffer, adds it to the chain of buffers of the curlHandle and stores the data received there"
	| dst byteSize |
	self var: #dataPtr		declareC: 'void *dataPtr'.
	self var: #items			declareC: 'unsigned items'.
	self var: #size			declareC: 'unsigned size'.
	self var: #curlHandle	declareC: 'curlStruct* curlHandle'.
	self var: #dst			declareC: 'void* dst'.
	self var: #byteSize		declareC: 'unsigned byteSize'.

	"create new data buffer for the curlHandle"
	byteSize := items*size.
	dst := curlHandle bufChainAddBuffer: byteSize.
	dst = 0 ifTrue: [^0]. "something wrong - curl lib should notice the returned zero instead of items*size"

	self cCode:  'memcpy( dst, dataPtr, items*size )'.
	"increment data byte count"
	curlHandle dataSizeSet: (byteSize + curlHandle dataSizeGet).
	^items*size
	
	! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

CurlPlugin class
	instanceVariableNames: ''!
!CurlPlugin class commentStamp: '<historical>' prior: 0!
CurlHandle is a C-side plugin structure keeping various information associated with the  curl_easy_handle.
This information is maintaned by the plugin code. 
Image side requests all services from the plugin passing aLargeIntegerCurlHandle (representing pointer to CurlHandle) to it.
So here and later in the code, convention is as follows:
	CurlHandle		- refers to plugin handle (passed between smalltalk and plugin)
	EasyHandle		- refers to libcurl 'easy' handle (passed between plugin and libcurl)

CurlHandle is organized as a C-structures (see CurlPlugin class >> curlStructDeclaration)
Each download results in a chain of data-buffers filled with data ((see CurlPlugin class >> dataBufferStructDeclaration))
!


!CurlPlugin class methodsFor: 'plugin generation' stamp: 'dao 9/24/2006 17:49'!
declareCVarsIn: cg 

	cg addHeaderFile: '<curl/curl.h>' 
				, '\' withCRs , 

	" (Note for that strange garbage above ;-) )
	  We abuse code generator by appending our C-definitions to the name of included file.
	  Now we have an opportunity to say all we want in plain-C :)
	  When choosing between doing a little trick on cg and necessity to maintain separate files - I'm choosing the first"

	self inlineCurlPluginCDeclarations.


	"normal cg declaration"
	cg var: #memChunks 	declareC: 'int memChunks'.
	cg var: #curlHandleFirst	declareC: 'curlStruct* curlHandleFirst'.
	cg var: #curlHandleLast		declareC: 'curlStruct* curlHandleLast'.
	cg var: #curlShare		declareC: 'CURLSH *curlShare'.

! !

!CurlPlugin class methodsFor: 'plugin generation' stamp: 'dao 8/30/2006 11:57'!
moduleName

	^'CurlPlugin'! !


!CurlPlugin class methodsFor: 'custom C-definitions' stamp: 'dao 10/8/2006 17:54'!
curlStructDeclaration

	^'
typedef struct   {
	struct curlStruct*	next;			/* pointer to the next instance or 0 if last */
	struct curlStruct*	prev;
	CURL*			easyHandle;			/* pointer to libcurl easy handle */
	CURLM*			multiHandle;		/* pointer to libcurl multi handle */
	CURLcode		easyStatus;			/* status of the last performed by easy handle operation */
	CURLMcode		multiStatus;			/* status of the last performed by multi handle operation */
	int				isMultiRunning;		/* count of transfers in progress (0/1 in our case) */
	char*			url;					/* pointer to \0 terminated string, which MUST remain present (it is not copied by libcurl) */
	char*			userPwd;			/* \0-string which should be [user name]:[password] to use for the connection */
	char*			cainfo;				/* SSL related, path to the sertificates files  */
	char*			proxy;				/* \0-string holding the host name or dotted IP address */
	char*			proxyUserPwd;		
	char*			cookie;
	char*			cookieFile;
	char*			cookieJar;		
	unsigned long	dataSize;			/* Bytes allready in buffer */
	unsigned		numberOfBuf;		/* Number of buffers allready allocated */
	unsigned		readPosition;		/* Current read position in the current buffer */
	void*			readPositionBuf;		/* Buffer where read cursor is */
	dataBufferStruct*	dataBufHead;		/* Pointer to the first data buffer (see below) structure */
	dataBufferStruct*	dataBufTail;			/* Pointer to the last data buffer structure */
	} curlStruct;
 '! !

!CurlPlugin class methodsFor: 'custom C-definitions' stamp: 'dao 9/16/2006 14:03'!
dataBufferStructDeclaration

	^'
typedef struct  {

	unsigned long				size;			/* buffer byte size */
	struct dataBufferStruct*		next;			/* next buffer or 0 if last */
	struct dataBufferStruct*		prev;			/* previous buffer or 0 if first */
	int							data;			/* data starts here */
	} dataBufferStruct;
 '! !

!CurlPlugin class methodsFor: 'custom C-definitions' stamp: 'dao 9/16/2006 14:00'!
inlineCurlPluginCDeclarations
	

	^self  macSpecificDefines , self dataBufferStructDeclaration , self curlStructDeclaration ! !

!CurlPlugin class methodsFor: 'custom C-definitions' stamp: 'dao 9/16/2006 13:29'!
macSpecificDefines

	^'
/* MAC-OS memory mgmt aliases */

#ifdef TARGET_OS_MAC
#define	malloc(size) NewPtr(size)
#define free(ptr) DisposePtr(ptr)
#endif
'! !
